export function debounce<T extends (...args: any[]) => any>(
	fn: T,
	delay = 200,
): T {
	let timeoutId: number | undefined;

	return function (this: any, ...args: Parameters<T>): void {
		clearTimeout(timeoutId);

		timeoutId = setTimeout(() => {
			fn.apply(this, args);
		}, delay);
	} as T;
}
